@@ -254,10 +254,14 @@ getFactTable <- function(x, table, sender="%in%") {
 setMethod("Compare", signature(e1="SpatRaster", e2="character"),
     function(e1, e2){
 		oper <- as.vector(.Generic)[1]
-		e2 <- getCatIDs(e1, e2, "==")
 		if (oper != "==") {
 			error(oper, "only '==' is supported with categorical comparisons")
 		}
+		if (nlyr(e1) > 1) {
+			return(rast(lapply(e1, function(x) x == e2)))
+		}
+
+		e2 <- getCatIDs(e1, e2, "==")
 		if (length(e2) == 0) {
 			return(as.logical(e1*0))
 		}
@@ -368,7 +372,13 @@ setMethod("is.bool", signature(x="SpatRaster"),
 )
 setMethod("is.int", signature(x="SpatRaster"),
 	function(x) {
-		x@pntr$valueType(FALSE) == 1
+		(x@pntr$valueType(FALSE) == 1) & (!x@pntr$hasCategories())
+	}
+)
+
+setMethod("is.num", signature(x="SpatRaster"),
+	function(x) {
+		(x@pntr$valueType(FALSE) < 2) & (!x@pntr$hasCategories())
 	}
 )
 
@@ -728,7 +738,7 @@ setMethod("logic", signature(x="SpatRaster"),
 			error("logic", "oper must be a character value")
 		}
 		oper = oper[1]
-		ops <- c("!", "is.na", "allNA", "noNA", "is.infinite", "is.finite", "iSTRUE", "isFALSE")
+		ops <- c("!", "is.na", "allNA", "noNA", "is.infinite", "is.finite", "isTRUE", "isFALSE")
 		if (!(oper %in% ops)) {
 			error("compare", "oper must be a one of", ops)		
 		}
