@@ -692,7 +692,10 @@ setMethod("project", signature(x="SpatRaster"),
 			}
 		} else {
 			if (!is.character(y)) {
-				warn("project,SpatRaster", "argument y (the crs) should be a character value")
+				#warn("project,SpatRaster", "argument y (the crs) should be a character value")
+				if (inherits(y, "numeric")) {
+					error("project,SpatRaster", "argument y (the crs) cannot be a number.\nFor EPSG codes use this format 'epsg:1234'")				
+				}
 				y <- as.character(crs(y))
 			}
 			if (!is.null(res) || !is.null(origin)) {
@@ -819,18 +822,53 @@ setMethod("rectify", signature(x="SpatRaster"),
 	}
 )
 
+
+resample_method <- function(x, method) {
+	if (missing(method)) {
+		if (is.factor(x)[1] || isTRUE(x@pntr$rgb)) {
+			method <- "near"
+		} else {
+			method <- "bilinear"
+		}
+	} else {
+		mts <- c("near", "bilinear", "cubic", "cubicspline", "lanczos", "mean", "sum", "modal", "min", "q1", "median", "q3", "max", "rms")
+		if (!inherits(method, "character")) {
+			method <- .makeTextFun(method)
+			if (!inherits(method, "character")) {
+				error("resample", paste("Not a valid method. Use one of:", paste(mts, collapse=", ")))
+			}
+		}
+		method <- match.arg(tolower(method[1]), c(mts, "average", "mode"))
+		method[method == "mean"] <- "average"
+		method[method == "modal"] <- "mode"
+	}
+	method
+}
+	
+
+setMethod("resample", signature(x="SpatRaster", y="numeric"),
+	function(x, y, method, threads=FALSE, by_util=FALSE, filename="", ...)  {
+
+		stopifnot(all(y > 0))
+		stopifnot(length(y) <= 2)
+		fact <- rep_len(y, 2)
+		r <- rast(x)
+		res(r) <- res(r) * fact
+		r <- crop(extend(r, c(0,1,0,1)), ext(x), snap="out") 
+
+		if (!hasValues(x)) { 
+			return(r)
+		}	
+		method <- resample_method(x, method)
+		resample(x, r, method, threads=threads, by_util=by_util, filename=filename, ...)
+	}
+)
+
+
 setMethod("resample", signature(x="SpatRaster", y="SpatRaster"),
 	function(x, y, method, threads=FALSE, by_util=FALSE, filename="", ...)  {
 
-		if (missing(method)) {
-			if (is.factor(x)[1] || isTRUE(x@pntr$rgb)) {
-				method <- "near"
-			} else {
-				method <- "bilinear"
-			}
-		} else {
-			method <- match.arg(tolower(method[1]), c("near", "bilinear", "cubic", "cubicspline", "lanczos", "average", "sum", "mode", "min", "q1", "median", "q3", "max", "rms"))			
-		}
+		method <- resample_method(x, method)
 		xcrs = crs(x)
 		ycrs = crs(y)
 		if ((xcrs == "") && (ycrs != "")) {
@@ -873,9 +911,9 @@ setMethod("rev", signature(x="SpatRaster"),
 )
 
 setMethod("rotate", signature(x="SpatRaster"),
-	function(x, left=TRUE, filename="", ...) {
+	function(x, filename="", ...) {
 		opt <- spatOptions(filename, ...)
-		x@pntr <- x@pntr$rotate(left, opt)
+		x@pntr <- x@pntr$rotate(TRUE, opt)
 		messages(x, "rotate")
 	}
 )
@@ -1161,7 +1199,11 @@ setMethod("unique", signature(x="SpatRaster", incomparables="ANY"),
 
 		opt <- spatOptions()
 
-		if (as.raster) incomparables = FALSE
+		if (as.raster) {
+			if (nlyr(x) == 1) return(x)
+			incomparables = FALSE
+		}
+		
 		u <- x@pntr$unique(incomparables, digits, na.rm[1], opt)
 
 		if (!as.raster) {
