@@ -20,18 +20,19 @@ setMethod("sel", signature(x="SpatRaster"),
 
 
 setMethod("sel", signature(x="SpatVector"),
-	function(x, use="rec", draw=TRUE, col="cyan", ...) {
+	function(x, use="rec", show=TRUE, col="cyan", draw=TRUE, ...) {
 		use <- substr(tolower(use), 1, 3)
 		use <- match.arg(use, c("rec", "pol"))
+		scol <- ifelse(draw, "red", NA)
 		if (use == "rec") {
-			e <- draw()
+			e <- draw(col=scol)
 		#	e <- as.polygons(e)
 		} else {
-			e <- draw("pol")
+			e <- draw("pol", col=scol)
 		}
 		i <- is.related(x, e, "intersects")
 		x <- x[i, ]
-		if (draw) {
+		if (show) {
 			if (geomtype(x) == "points" || geomtype(x) == "multipoints") {
 				points(x, col=col, ...)
 			} else {
