@@ -3,21 +3,22 @@
 setMethod("blocks", signature(x="SpatRaster"),
 	function(x, n=4) {
 		opt <- spatOptions("", FALSE, ncopies=n)
-		b <- x@cpp$getBlockSizeR(n, opt$memfrac)
+		b <- x@pntr$getBlockSizeR(opt)
 		b$row <- b$row + 1
 		b
 	}
 )
 
 
+
 setMethod("writeStart", signature(x="SpatRaster", filename="character"),
 	function(x, filename="", overwrite=FALSE, n=4, sources="", ...) {
 		filename <- path.expand(trimws(filename[1]))
 		filename <- enc2utf8(filename)
 		opt <- spatOptions(filename, overwrite, ncopies=n, ...)
-		ok <- x@cpp$writeStart(opt, unique(sources))
+		ok <- x@pntr$writeStart(opt, unique(sources))
 		messages(x, "writeStart")
-		b <- x@cpp$getBlockSizeWrite()
+		b <- x@pntr$getBlockSizeWrite()
 		b$row <- b$row + 1
 		b
 	}
@@ -26,7 +27,7 @@ setMethod("writeStart", signature(x="SpatRaster", filename="character"),
 
 setMethod("writeStop", signature(x="SpatRaster"),
 	function(x) {
-		success <- x@cpp$writeStop()
+		success <- x@pntr$writeStop()
 		messages(x, "writeStop")
 		f <- sources(x)
 		if (f != "") {
@@ -38,7 +39,7 @@ setMethod("writeStop", signature(x="SpatRaster"),
 
 setMethod("writeValues", signature(x="SpatRaster", v="vector"),
 	function(x, v, start, nrows) {
-		success <- x@cpp$writeValues(v, start-1, nrows)
+		success <- x@pntr$writeValues(v, start-1, nrows)
 		messages(x, "writeValues")
 		invisible(success)
 	}
@@ -63,24 +64,26 @@ function(x, filename="", overwrite=FALSE, ...) {
 	#	return(saveRDS(x, filename))
 	#}
 	opt <- spatOptions(filename, overwrite, ...)
-	x@cpp <- x@cpp$writeRaster(opt)
+	x@pntr <- x@pntr$writeRaster(opt)
 	x <- messages(x, "writeRaster")
 	invisible(rast(filename))
 }
 )
 
 
-get_filetype <- function(filename) {
-	ext <- tolower(tools::file_ext(filename))
-	if (ext == "shp" || ext == "") {
+get_filetype <- function(filename) {  
+	fn <- tolower(filename)
+	ext <- tools::file_ext(fn)
+	ext2 <- gsub("^[^.]*\\.(.*).*$", "\\1", fn)
+	if ((ext == "shp") || (ext == "shz") || (ext == "zip" && ext2 == "shp.zip")) {
 		"ESRI Shapefile"
-	} else if (ext == "gpkg") {
+	} else if ((ext == "") || (ext == "gpkg") || (ext == "zip" && ext2 == "gpkg.zip")) {
 		"GPKG"
 	} else if (ext == "gdb") {
 		"OpenFileGDB"
 	} else if (ext == "gml") {
 		"GML"
-	} else if (ext == "json") {
+	} else if ((ext == "json") || (ext == "geojson")) {
 		"GeoJSON"
 	} else if (ext == "cdf") {
 		"netCDF"
@@ -132,11 +135,11 @@ function(x, filename, filetype=NULL, layer=NULL, insert=FALSE, overwrite=FALSE,
 				newnms[j] <- paste0(newnms[j], "0")
 				nms[i] <- newnms
 			}
-			x@cpp <- x@cpp$deepcopy()
+			x@pntr <- x@pntr$deepcopy()
 			names(x) <- nms
 		}
 	}
-	success <- x@cpp$write(filename, layer, filetype, insert[1], overwrite[1], options)
+	success <- x@pntr$write(filename, layer, filetype, insert[1], overwrite[1], options)
 	messages(x, "writeVector")
 	invisible(TRUE)
 }
@@ -173,11 +176,11 @@ function(x, filename, filetype=NULL, layer=NULL, insert=FALSE, overwrite=FALSE,
 				# newnms[j] <- paste0(newnms[j], "0")
 				# nms[i] <- newnms
 			# }
-			# x@cpp <- x@cpp$deepcopy()
+			# x@pntr <- x@pntr$deepcopy()
 			# names(x) <- nms
 		# }
 	# }
-	# success <- x@cpp$write_proxy(filename, layer, filetype, insert[1], FALSE, overwrite[1], options)
+	# success <- x@pntr$write_proxy(filename, layer, filetype, insert[1], FALSE, overwrite[1], options)
 	# messages(x, "writeVector")
 	# invisible(TRUE)
 # }
