@@ -1,4 +1,4 @@
-// Copyright (c) 2018-2023  Robert J. Hijmans
+// Copyright (c) 2018-2025  Robert J. Hijmans
 //
 // This file is part of the "spat" library.
 //
@@ -22,6 +22,13 @@
 #include "vecmath.h"
 #include <cmath>
 
+#if defined(USE_TBB)
+#include <tbb/tbb.h>
+#include <tbb/parallel_for.h>
+#include <tbb/blocked_range.h>
+#endif 
+
+
 //#include "modal.h"
 
 /*
@@ -51,6 +58,21 @@ void operator*(std::vector<double>& a, const std::vector<double>& b) {
 
 */
 
+
+
+inline double R_modulo(const double &x, const double &y) { 
+	double m = std::fmod(x, y);
+	if (y < 0) {
+		if (m > 0) {
+			m += y; 
+		}
+	} else if (m < 0) {
+        m += y; 
+    }
+	return m;
+}
+
+
 //template <typename T>
 void operator%(std::vector<double>& a, const std::vector<double>& b) {
 //    std::transform(a.begin(), a.end(), b.begin(), a.begin(), std::modulus<T>());
@@ -58,7 +80,7 @@ void operator%(std::vector<double>& a, const std::vector<double>& b) {
 		if (std::isnan(a[i]) || std::isnan(b[i])) {
 			a[i] = NAN;
 		} else {
-			a[i] = std::fmod(a[i], b[i]);
+			a[i] = R_modulo(a[i], b[i]);
 		}
 	}
 }
@@ -143,7 +165,7 @@ bool smooth_operator(std::string &oper, bool &logical, bool &reverse, bool &fals
 }
 
 
-
+//#include <execution>
 
 SpatRaster SpatRaster::arith(SpatRaster x, std::string oper, bool falseNA, SpatOptions &opt) {
 
@@ -201,11 +223,14 @@ SpatRaster SpatRaster::arith(SpatRaster x, std::string oper, bool falseNA, SpatO
 		return out;
 	}
 
+//	auto policy = std::execution::par;
+
 	for (size_t i = 0; i < out.bs.n; i++) {
 		std::vector<double> a, b;
 		readBlock(a, out.bs, i);
 		x.readBlock(b, out.bs, i);
 		recycle(a,b);
+		
 		if (oper == "+") {
 			std::transform(a.begin(), a.end(), b.begin(), a.begin(), std::plus<double>());
 		} else if (oper == "-") {
@@ -226,7 +251,7 @@ SpatRaster SpatRaster::arith(SpatRaster x, std::string oper, bool falseNA, SpatO
 			 a % b;
 		} else if (oper == "%/%") {
 			for (size_t i=0; i<a.size(); i++) {
-				a[i] = trunc(a[i] / b[i]);
+				a[i] = floor(a[i] / b[i]);
 			}
 		} else if (oper == "==") {
 			for (size_t i=0; i<a.size(); i++) {
@@ -268,7 +293,7 @@ SpatRaster SpatRaster::arith(SpatRaster x, std::string oper, bool falseNA, SpatO
 
 SpatRaster SpatRaster::arith(double x, std::string oper, bool reverse, bool falseNA, SpatOptions &opt) {
 
-	SpatRaster out = geometry(nlyr());
+	SpatRaster out = geometry();
 	if (!hasValues()) {
 		out.setError("raster has no values"); // or warn and treat as NA?
 		return out;
@@ -309,8 +334,10 @@ SpatRaster SpatRaster::arith(double x, std::string oper, bool reverse, bool fals
 		std::vector<double> a;
 		readBlock(a, out.bs, i);
 		if (std::isnan(x)) {
-			for(double& d : a)  d = NAN;
+			std::fill(a.begin(), a.end(), NAN);
+			//for (double& d : a)  d = NAN;
 		} else if (oper == "+") {
+//			std::for_each(std::execution::par, a.begin(), a.end(), [&](double& d) {	d += x;	});
 			for (double& d : a)  d += x;
 		} else if (oper == "-") {
 			if (reverse) {
@@ -336,18 +363,18 @@ SpatRaster SpatRaster::arith(double x, std::string oper, bool reverse, bool fals
 		} else if (oper == "%") {
 			if (reverse) {
 				for (size_t i=0; i<a.size(); i++) {
-					a[i] = std::fmod(x, a[i]);
+					a[i] = R_modulo(x, a[i]);
 				}
 			} else {
 				for (size_t i=0; i<a.size(); i++) {
-					a[i] = std::fmod(a[i], x);
+					a[i] = R_modulo(a[i], x);
 				}
 			}
 		} else if (oper == "%/%") {
 			if (reverse) {
-				for (double& d : a) d = trunc(x / d);
+				for (double& d : a) d = floor(x / d);
 			} else {
-				for (double& d : a) d = trunc(d / x);
+				for (double& d : a) d = floor(d / x);
 			}
 		} else if (oper == "==") {
 			for (double& d : a) if (!std::isnan(d)) d = d == x;
@@ -402,8 +429,8 @@ SpatRaster SpatRaster::arith(std::vector<double> x, std::string oper, bool rever
 		return(arith(x[0], oper, reverse, falseNA, opt));
 	}
 
-	unsigned innl = nlyr();
-	unsigned outnl = innl;
+	size_t innl = nlyr();
+	size_t outnl = innl;
 
 	if (x.size() > innl) {
 		outnl = x.size();
@@ -446,7 +473,7 @@ SpatRaster SpatRaster::arith(std::vector<double> x, std::string oper, bool rever
 		return out;
 	}
 
-	unsigned nc = ncol();
+	size_t nc = ncol();
 	recycle(x, outnl);
 
 	for (size_t i = 0; i < out.bs.n; i++) {
@@ -456,9 +483,9 @@ SpatRaster SpatRaster::arith(std::vector<double> x, std::string oper, bool rever
 			recycle(v, outnl * out.bs.nrows[i] * nc);
 		}
 		//std::vector<double> vv;
-		unsigned off = out.bs.nrows[i] * nc;
+		size_t off = out.bs.nrows[i] * nc;
 		for (size_t j=0; j<outnl; j++) {
-			unsigned s = j * off;
+			size_t s = j * off;
 			if (std::isnan(x[j])) {
 				for (size_t k=0; k<off; k++) {
 					v[s+k] = NAN;
@@ -504,11 +531,11 @@ SpatRaster SpatRaster::arith(std::vector<double> x, std::string oper, bool rever
 			} else if (oper == "%") {
 				if (reverse) {
 					for (size_t k=0; k<off; k++) {
-						v[s+k] = std::fmod(x[j], v[s+k]);
+						v[s+k] = R_modulo(x[j], v[s+k]);
 					}
 				} else {
 					for (size_t k=0; k<off; k++) {
-						v[s+k] = std::fmod(v[s+k], x[j]);
+						v[s+k] = R_modulo(v[s+k], x[j]);
 					}
 				}
 			} else if (oper == "==") {
@@ -581,7 +608,7 @@ SpatRaster SpatRaster::arith_m(std::vector<double> x, std::string oper, std::vec
 	if (dim[0] < 2) {
 		return(arith(x, oper, reverse, false, opt));
 	}
-	if (dim[0] > ncell()) {
+	if ((double) dim[0] > ncell()) {
 		out.setError("incorrect matrix dimensions (nrow > ncell(x))"); 
 		return out;
 	}
@@ -635,12 +662,12 @@ SpatRaster SpatRaster::arith_m(std::vector<double> x, std::string oper, std::vec
 		return out;
 	}
 
-	unsigned nc = ncol();
+	size_t nc = ncol();
 	
 	for (size_t i = 0; i < out.bs.n; i++) {
 		std::vector<double> v;
 		readBlock(v, out.bs, i);
-		unsigned off = out.bs.nrows[i] * nc;
+		size_t off = out.bs.nrows[i] * nc;
 		for (size_t j=0; j<nl; j++) {
 			size_t s = j * off;
 			size_t d = j * dim[0];
@@ -692,11 +719,11 @@ SpatRaster SpatRaster::arith_m(std::vector<double> x, std::string oper, std::vec
 			} else if (oper == "%") {
 				if (reverse) {
 					for (size_t k=0; k<off; k++) {
-						v[s+k] = std::fmod(xj[k], v[s+k]);
+						v[s+k] = R_modulo(xj[k], v[s+k]);
 					}
 				} else {
 					for (size_t k=0; k<off; k++) {
-						v[s+k] = std::fmod(v[s+k], xj[k]);
+						v[s+k] = R_modulo(v[s+k], xj[k]);
 					}
 				}
 			} else if (oper == "==") {
@@ -745,6 +772,7 @@ double dabs(double x) {
 	return (x < 0 ? -1 * x : x);
 }
 
+
 SpatRaster SpatRaster::math(std::string fun, SpatOptions &opt) {
 
 	SpatRaster out = geometry();
@@ -797,10 +825,25 @@ SpatRaster SpatRaster::math(std::string fun, SpatOptions &opt) {
 		readStop();
 		return out;
 	}
+
 	for (size_t i = 0; i < out.bs.n; i++) {
 		std::vector<double> a;
 		readBlock(a, out.bs, i);
-		for(double& d : a) if (!std::isnan(d)) d = mathFun(d);
+
+#if defined(USE_TBB)
+		if (opt.parallel) {
+			tbb::parallel_for(tbb::blocked_range<size_t>(0, a.size()),
+				[&](const tbb::blocked_range<size_t>& range) {
+				for (size_t i = range.begin(); i != range.end(); i++) {
+					if (!std::isnan(a[i])) a[i] = mathFun(a[i]);
+				}
+			});
+		} else {
+			for (double& d : a) if (!std::isnan(d)) d = mathFun(d);
+		}
+#else
+		for (double& d : a) if (!std::isnan(d)) d = mathFun(d);	
+#endif
 		if (!out.writeBlock(a, i)) return out;
 	}
 	out.writeStop();
@@ -921,7 +964,23 @@ SpatRaster SpatRaster::trig(std::string fun, SpatOptions &opt) {
 	for (size_t i = 0; i < out.bs.n; i++) {
 		std::vector<double> a;
 		readValues(a, out.bs.row[i], out.bs.nrows[i], 0, ncol());
+#if defined(USE_TBB) 
+		if (opt.parallel) {
+			tbb::parallel_for(tbb::blocked_range<size_t>(0, a.size()),
+				[&](const tbb::blocked_range<size_t>& range) {
+				for (size_t i = range.begin(); i != range.end(); i++) {
+					if (!std::isnan(a[i])) {
+						a[i] = trigFun(a[i]);
+					}
+				}
+			});
+		} else {
+			for (double& d : a) if (!std::isnan(d)) d = trigFun(d);
+		}
+#else 
 		for (double& d : a) if (!std::isnan(d)) d = trigFun(d);
+#endif	
+
 		if (!out.writeBlock(a, i)) return out;
 	}
 	out.writeStop();
@@ -1279,9 +1338,9 @@ SpatRaster SpatRaster::cum(std::string fun, bool narm, SpatOptions &opt) {
 		readStop();
 		return out;
 	}
-	unsigned nl = out.nlyr();
+	size_t nl = out.nlyr();
 	std::vector<double> v(nl);
-	unsigned nc;
+	size_t nc;
 	for (size_t i = 0; i < out.bs.n; i++) {
 		std::vector<double> a;
 		readBlock(a, out.bs, i);
@@ -1352,14 +1411,14 @@ SpatRaster SpatRaster::summary_numb(std::string fun, std::vector<double> add, bo
 		readStop();
 		return out;
 	}
-	unsigned nl = nlyr();
+	size_t nl = nlyr();
 	std::vector<double> v(nl);
 	if (!add.empty()) v.insert( v.end(), add.begin(), add.end() );
 
 	for (size_t i = 0; i < out.bs.n; i++) {
 		std::vector<double> a;
 		readBlock(a, out.bs, i);
-		unsigned nc = out.bs.nrows[i] * out.ncol();
+		size_t nc = out.bs.nrows[i] * out.ncol();
 		std::vector<double> b(nc);
 		for (size_t j=0; j<nc; j++) {
 			for (size_t k=0; k<nl; k++) {
@@ -1413,20 +1472,20 @@ SpatRaster SpatRaster::modal(std::vector<double> add, std::string ties, bool nar
 	std::default_random_engine rgen(seed);
 	std::uniform_real_distribution<double> dist (0.0,1.0);
 
-	unsigned nl = nlyr();
+	size_t nl = nlyr();
 	std::vector<double> v(nl);
 	v.insert( v.end(), add.begin(), add.end() );
 
 	for (size_t i = 0; i < out.bs.n; i++) {
 		std::vector<double> a;
 		readBlock(a, out.bs, i);
-		unsigned nc = out.bs.nrows[i] * out.ncol();
+		size_t nc = out.bs.nrows[i] * out.ncol();
 		std::vector<double> b(nc);
 		for (size_t j=0; j<nc; j++) {
 			for (size_t k=0; k<nl; k++) {
 				v[k] = a[j+k*nc];
 			}
-			b[j] = modal_value(v, ities, narm, rgen, dist);
+			b[j] = modal_value(v, (unsigned)ities, narm, rgen, dist);
 		}
 		if (!out.writeBlock(b, i)) return out;
 	}
@@ -1454,14 +1513,14 @@ SpatRaster SpatRaster::range(std::vector<double> add, bool narm, SpatOptions &op
 		readStop();
 		return out;
 	}
-	unsigned nl = nlyr();
+	size_t nl = nlyr();
 	std::vector<double> v(nl);
 	v.insert( v.end(), add.begin(), add.end() );
 
 	for (size_t i = 0; i < out.bs.n; i++) {
 		std::vector<double> a;
 		readBlock(a, out.bs, i);
-		unsigned nc = out.bs.nrows[i] * out.ncol();
+		size_t nc = out.bs.nrows[i] * out.ncol();
 		std::vector<double> b(nc * 2);
 		for (size_t j=0; j<nc; j++) {
 			for (size_t k=0; k<nl; k++) {
@@ -1483,10 +1542,10 @@ SpatRaster SpatRaster::range(std::vector<double> add, bool narm, SpatOptions &op
 
 SpatRaster SpatRasterStack::summary_numb(std::string fun, std::vector<double> add, bool narm, SpatOptions &opt) {
 
-	std::vector<unsigned> vnl = nlyr();
-	unsigned nl = vmax(vnl, false);
+	std::vector<size_t> vnl = nlyr();
+	size_t nl = vmax(vnl, false);
 	SpatRaster out = ds[0].geometry(nl);
-	unsigned ns = nsds();
+	size_t ns = nsds();
 
 	if (fun == "range") {
 		out.setError("parallel range not implemented, use min and max");
@@ -1522,7 +1581,7 @@ SpatRaster SpatRasterStack::summary_numb(std::string fun, std::vector<double> ad
 
 	std::vector<std::vector<double>> a(ns);
 	for (size_t i=0; i < out.bs.n; i++) {
-		unsigned nc = out.bs.nrows[i] * out.ncol() * nl;
+		size_t nc = out.bs.nrows[i] * out.ncol() * nl;
 		for (size_t j=0; j < ns; j++) {
 			ds[j].readBlock(a[j], out.bs, i);
 			recycle(a[j], nc);
@@ -1884,7 +1943,7 @@ SpatRaster SpatRaster::isinfinite(bool falseNA, SpatOptions &opt) {
 
 std::vector<std::vector<double>> SpatRaster::where(std::string what, bool values, SpatOptions &opt) {
 
-	unsigned nl = nlyr();
+	size_t nl = nlyr();
 	std::vector<std::vector<double>> out(nl);
 
 	std::vector<std::string> f {"min", "max"};
@@ -1906,28 +1965,10 @@ std::vector<std::vector<double>> SpatRaster::where(std::string what, bool values
 	std::vector<double> val;
 	bool do_min = what == "min";
 
-	std::vector<bool> hr = hasRange();
-	bool hasR = true;
-	for (size_t i=0; i<hr.size(); i++) {
-		if (!hr[i]) {
-			hasR = false;
-			break;
-		}
-	}
-	if (hasR) {
-		if (do_min) {
-			val = range_min();
-			for (double &d : val) d *= 1.00001;
-		} else {
-			val = range_max();
-			for (double &d : val) d *= 0.99999;
-		}
+	if (do_min) {
+		val.resize(nl, std::numeric_limits<double>::max());
 	} else {
-		if (do_min) {
-			val.resize(nl, std::numeric_limits<double>::max());
-		} else {
-			val.resize(nl, std::numeric_limits<double>::lowest());
-		}
+		val.resize(nl, std::numeric_limits<double>::lowest());
 	}
 	for (size_t i=0; i<bs.n; i++) {
 		std::vector<double> v;
@@ -1942,10 +1983,10 @@ std::vector<std::vector<double>> SpatRaster::where(std::string what, bool values
 						if (v[k] < val[j]) {
 							val[j] = v[k];
 							out[j].resize(0);
-							double cell = k - off + boff;
+							double cell = (double) (k - off + boff);
 							out[j].push_back(cell);
 						} else if (v[k] == val[j]) {
-							double cell = k - off + boff;
+							double cell = (double) (k - off + boff);
 							out[j].push_back(cell);
 						}
 					}
@@ -1956,22 +1997,26 @@ std::vector<std::vector<double>> SpatRaster::where(std::string what, bool values
 						if (v[k] > val[j]) {
 							val[j] = v[k];
 							out[j].resize(0);
-							double cell = k - off + boff;
+							double cell = (double) (k - off + boff);
 							out[j].push_back(cell);
 						} else if (v[k] == val[j]) {
-							double cell = k - off + boff;
+							double cell = (double) (k - off + boff);
 							out[j].push_back(cell);
 						}
 					}
 				}
 			}
-			if (values) {
-				std::vector<double> wval(out[j].size(), val[j]);
-				out[j].insert(out[j].end(), wval.begin(), wval.end());
-			}
 		}
 	}
 	readStop();
+
+	if (values) {
+		for (size_t j=0; j<nl; j++) {
+			std::vector<double> wval(out[j].size(), val[j]);
+			out[j].insert(out[j].end(), wval.begin(), wval.end());
+		}
+	}
+
 	return(out);
 }
 
