@@ -286,8 +286,8 @@ SpatVector SpatVector::set_precision(double gridSize) {
 */
 
 
-std::vector<std::vector<unsigned>> SpatVector::index_2d(SpatVector v) {
-	std::vector<std::vector<unsigned>> out(2);
+std::vector<std::vector<size_t>> SpatVector::index_2d(SpatVector v) {
+	std::vector<std::vector<size_t>> out(2);
 	size_t n = std::max(size(), v.size()) * 2;
 	out[0].reserve(n);
 	out[1].reserve(n);
@@ -310,8 +310,8 @@ std::vector<std::vector<unsigned>> SpatVector::index_2d(SpatVector v) {
 }
 
 
-std::vector<std::vector<unsigned>> SpatVector::index_sparse(SpatVector v) {
-	std::vector<std::vector<unsigned>> out(v.size());
+std::vector<std::vector<size_t>> SpatVector::index_sparse(SpatVector v) {
+	std::vector<std::vector<size_t>> out(v.size());
 	for (size_t i=0; i<size(); i++) {
 		for (size_t j=0; j<size(); j++) {
 			if (geoms[i].extent.intersects(v.geoms[j].extent)) {
@@ -722,6 +722,11 @@ SpatVector SpatVector::shared_paths(SpatVector x, bool index) {
 }
 
 
+#ifndef GEOS370
+bool find_segments(GEOSContextHandle_t hGEOSCtxt, std::vector<double> &x, std::vector<double> &y, std::vector<double> &cx, std::vector<double> &cy, std::vector<size_t> &si, std::vector<double> &sx, std::vector<double> &sy) {
+	return false;
+}	
+#else
 bool find_segments(GEOSContextHandle_t hGEOSCtxt, std::vector<double> &x, std::vector<double> &y, std::vector<double> &cx, std::vector<double> &cy, std::vector<size_t> &si, std::vector<double> &sx, std::vector<double> &sy) {
 	size_t n = x.size() - 1;
 	size_t m = cx.size() - 1;
@@ -740,36 +745,40 @@ bool find_segments(GEOSContextHandle_t hGEOSCtxt, std::vector<double> &x, std::v
 	}
 	return si.size() > 0;
 }
-
+#endif
 
 SpatVector SpatVector::split_lines(SpatVector v) {
 
 // check GEOS version 
 
 	SpatVector out;
+	#ifndef GEOS370
+	out.setError("not available with GEOS < 3.7");
 	return out;
+	#endif
 
 /*
-	SpatVector out = *this;
 	std::vector<size_t> si;
 	std::vector<double> sx, sy;
 	GEOSContextHandle_t hGEOSCtxt = geos_init();
 
 	for (size_t i=0; i<v.size(); i++) {
 		SpatVector tmp = v.subset_rows(i);
-		std::vector<int> x = out.relate(tmp, "intersects", true, true);
+		std::vector<int> x = relate(tmp, "intersects", true, true);
 		std::vector<std::vector<double>> xy1 = tmp.coordinates();
 		for (size_t j=0; j<x.size(); j++) {
 			if (x[j] == 1) {
 				std::vector<std::vector<double>> xy2 = tmp.coordinates();
-//				if (find_segments(hGEOSCtxt, xy1[0], xy1[1], xy2[0], xy2[1], si, sx, sy)) {
+				if (find_segments(hGEOSCtxt, xy1[0], xy1[1], xy2[0], xy2[1], si, sx, sy)) {
 					
-//				}
+				}
 			}
 		}
 	}
+	
+*/	
 	return out;
-*/
+
 }	
 
 
@@ -821,7 +830,11 @@ SpatVector SpatVector::polygonize() {
 	}
 
 	SpatVector out;
-	out.srs = srs;
+	if (type() == "points") {
+		out.setError("cannot make polygons from points (make lines first)");
+		return out;
+	}
+
 	GEOSContextHandle_t hGEOSCtxt = geos_init();
 
 	std::vector<GeomPtr> g = geos_geoms(this, hGEOSCtxt);
@@ -1108,6 +1121,7 @@ SpatVector SpatVector::hull(std::string htype, std::string by, double param, boo
 
 
 
+
 SpatVector SpatVector::voronoi(SpatVector bnd, double tolerance, int onlyEdges) {
 	SpatVector out;
 
@@ -1156,7 +1170,7 @@ SpatVector SpatVector::voronoi(SpatVector bnd, double tolerance, int onlyEdges)
 		}
 		if ((type() == "points") && (!onlyEdges)) {
 			std::vector<int> atts = out.relateFirst(*this, "intersects");
-			std::vector<unsigned> a;
+			std::vector<size_t> a;
 			a.reserve(atts.size());
 			for (size_t i=0; i<atts.size(); i++) {
 				if (atts[i] >=0) a.push_back(atts[i]);
@@ -1353,15 +1367,15 @@ SpatVector SpatVector::intersect(SpatVector v, bool values) {
 	std::vector<GeomPtr> result;
 //	size_t nx = size();
 //	size_t ny = v.size();
-	std::vector<unsigned> idx, idy;
+	std::vector<size_t> idx, idy;
 
 	std::vector<std::vector<double>> r = which_relate(v, "intersects", true);
 	size_t n = r[0].size();
 	idx.reserve(n);
 	idy.reserve(n);
 	for (size_t i=0; i<n; i++) {
-		idx.push_back( (unsigned) r[0][i] );
-		idy.push_back( (unsigned) r[1][i] );
+		idx.push_back( r[0][i] );
+		idy.push_back( r[1][i] );
 	}
 	r.resize(0);
 	std::vector<long> ids;
@@ -1429,7 +1443,7 @@ SpatVector SpatVector::intersect(SpatVector v, bool values) {
 	n = out.nrow();
 	if (values) {
 		if (n < idx.size()) {
-			std::vector<unsigned> idx2, idy2;
+			std::vector<size_t> idx2, idy2;
 			idx2.reserve(n);
 			idy2.reserve(n);
 			for (size_t i=0; i<n; i++) {
@@ -1447,7 +1461,7 @@ SpatVector SpatVector::intersect(SpatVector v, bool values) {
 		}
 	} else {
 		if (n < idx.size()) {
-			std::vector<unsigned> idx2;
+			std::vector<size_t> idx2;
 			idx2.reserve(n);
 			for (size_t i=0; i<n; i++) {
 				idx2.push_back( idx[ out.df.iv[0][i] ]);
@@ -1457,6 +1471,9 @@ SpatVector SpatVector::intersect(SpatVector v, bool values) {
 			df1 = df.subset_rows(idx);
 		}
 	}
+	std::vector<std::string> nms = df1.get_names();
+	make_unique_names(nms);
+	df1.set_names(nms);	
 	out.df = df1;
 	return out;
 }
@@ -2331,8 +2348,8 @@ std::vector<bool> SpatVector::is_related(SpatVector v, std::string relation) {
 }
 
 
-std::vector<unsigned> SpatVector::equals_exact(SpatVector v, double tol) {
-	std::vector<unsigned> out;
+std::vector<size_t> SpatVector::equals_exact(SpatVector v, double tol) {
+	std::vector<size_t> out;
 	GEOSContextHandle_t hGEOSCtxt = geos_init();
 	std::vector<GeomPtr> x = geos_geoms(this, hGEOSCtxt);
 	std::vector<GeomPtr> y = geos_geoms(&v, hGEOSCtxt);
@@ -2349,8 +2366,8 @@ std::vector<unsigned> SpatVector::equals_exact(SpatVector v, double tol) {
 }
 
 
-std::vector<unsigned> SpatVector::equals_exact(bool symmetrical, double tol) {
-	std::vector<unsigned> out;
+std::vector<size_t> SpatVector::equals_exact(bool symmetrical, double tol) {
+	std::vector<size_t> out;
 	GEOSContextHandle_t hGEOSCtxt = geos_init();
 	std::vector<GeomPtr> x = geos_geoms(this, hGEOSCtxt);
 
@@ -2384,7 +2401,7 @@ SpatVector SpatVector::mask(SpatVector x, bool inverse) {
 			b[i] = !b[i];
 		}
 	}
-	std::vector<unsigned> r;
+	std::vector<size_t> r;
 	r.reserve(b.size());
 	for (size_t i=0; i<b.size(); i++) {
 		if (b[i]) r.push_back(i);
@@ -2615,8 +2632,16 @@ SpatVector SpatVector::symdif(SpatVector v) {
 	if (ve.hasError()) {
 		return ve;
 	}
-	out = out.append(ve, true);
-	return out;
+	if ((out.type() == "polygons") && (ve.type() == "polygons")) {
+		return out.append(ve, true);
+	} else if (ve.type() == "polygons") {
+		return ve;
+	} else if (out.type() == "polygons") {
+		return out;
+	} else {
+		SpatVector empty;
+		return empty;
+	} 
 
 /*
 	SpatVector out;
@@ -2702,7 +2727,7 @@ SpatVector SpatVector::erase_agg(SpatVector v) {
 
 	if ((type() == "points") || (v.type() == "points")) {
 		std::vector<bool> b = is_related(v, "intersects");
-		std::vector<unsigned> r;
+		std::vector<size_t> r;
 		r.reserve(b.size());
 		for (size_t i=0; i < b.size(); i++) {
 			if (!b[i]) r.push_back(i);
@@ -2722,7 +2747,7 @@ SpatVector SpatVector::erase_agg(SpatVector v) {
 // so we do
 	v = v.aggregate(true);
 	std::vector<GeomPtr> y = geos_geoms(&v, hGEOSCtxt);
-	std::vector<unsigned> rids;
+	std::vector<size_t> rids;
 	size_t nx = size();
 	std::vector<GeomPtr> result;
 
@@ -2767,7 +2792,7 @@ SpatVector SpatVector::erase(SpatVector v) {
 
 	if ((type() == "points") || (v.type() == "points")) {
 		std::vector<bool> b = is_related(v, "intersects");
-		std::vector<unsigned> r;
+		std::vector<size_t> r;
 		r.reserve(b.size());
 		for (size_t i=0; i<b.size(); i++) {
 			if (!b[i]) r.push_back(i);
@@ -2902,7 +2927,7 @@ SpatVector SpatVector::erase(bool sequential) {
 
 	GEOSContextHandle_t hGEOSCtxt = geos_init();
 	std::vector<GeomPtr> x = geos_geoms(this, hGEOSCtxt);
-	std::vector<unsigned> rids;
+	std::vector<size_t> rids;
 
 	if (sequential) {
 		for (size_t i = 0; i < (n-1); i++) {
@@ -3021,7 +3046,16 @@ SpatVector SpatVector::nearest_point(SpatVector v, bool parallel, const std::str
 	out.srs = srs;
 
 	
-	if (lonlat && (type() == "points") && (v.type() == "points")) {
+	if (lonlat) {
+		SpatVector x = *this;
+		if (x.type() != "points") {
+			x = x.densify(1000, false, false);
+			x = x.as_points(false);
+		}
+		if (v.type() != "points") {
+			v = v.densify(1000, false, false);
+			v = v.as_points(false);
+		}
 		std::vector<double> nlon, nlat, dist;
 		std::vector<long> id;
 		std::vector<std::vector<double>> p = coordinates();
@@ -3115,7 +3149,7 @@ SpatVector SpatVector::nearest_point(const std::string method) {
 	GEOSContextHandle_t hGEOSCtxt = geos_init();
 	std::vector<GeomPtr> x = geos_geoms(this, hGEOSCtxt);
 	std::vector<GeomPtr> b(n);
-	for (unsigned i = 0; i < n; i++) {
+	for (size_t i = 0; i < n; i++) {
 		SpatVector xa = remove_rows({i});
 		xa = xa.aggregate(false);
 		std::vector<GeomPtr> y = geos_geoms(&xa, hGEOSCtxt);
