@@ -1,4 +1,4 @@
-// Copyright (c) 2018-2023  Robert J. Hijmans
+// Copyright (c) 2018-2025  Robert J. Hijmans
 //
 // This file is part of the "spat" library.
 //
@@ -64,6 +64,8 @@ SpatTime_t get_time(long year, unsigned month, unsigned day, int hr, int min, in
 		}
 	}
 
+	if (month == 0) month = 6;
+	if (day == 0) day = 15;
 	time += (mdays[isleap(year)][month-1] + day) * 86400;
 	time += (hr * 3600) + (min * 60) + sec;
     return time;
@@ -123,7 +125,7 @@ std::vector<int> get_date(SpatTime_t x) {
 }
 
 
-
+/*
 std::vector<std::string> splitstr(std::string s, std::string delimiter){
 	std::vector<std::string> out;
 	size_t pos = 0;
@@ -134,6 +136,7 @@ std::vector<std::string> splitstr(std::string s, std::string delimiter){
 	out.push_back(s.substr(0, pos));
 	return out;
 }
+*/
 
 void replace_one_char(std::string& s, char from, char to) {
 	for (size_t i = 0; i < s.size(); i++) {
@@ -165,17 +168,17 @@ std::vector<int> getymd(std::string s) {
 	std::vector<std::string> x;
 	std::vector<std::string> y;
 	if (ncolon > 0) {
-		x = splitstr(s, " ");
+		x = strsplit(s, " ");
 		s = x[0];
 		if (x.size() > 1) {
 			x[1].erase(std::remove(x[1].begin(), x[1].end(), 'z'), x[1].end());
-			y = splitstr(x[1], ":");
+			y = strsplit(x[1], ":");
 		}
 	}
 
 	size_t ndash = std::count(s.begin(), s.end(), '-');
 	if (ndash == 2) {
-		x = splitstr(s, "-");
+		x = strsplit(s, "-");
 	}
 	x.insert(x.end(), y.begin(), y.end() );
 	std::vector<int> out(x.size());
@@ -347,19 +350,23 @@ SpatTime_t time_from_day_360(int syear, int smonth, int sday, double ndays) {
 SpatTime_t parse_time(std::string x) {
 	lrtrim(x);
 	std::vector<std::string> s = strsplit(x, " ");
+	if (s.size() == 1) {
+		s = strsplit(x, "T");
+	}
 
 	std::vector<std::string> time;
 	if ((!s[0].empty()) && (s[0].substr(0, 1) != "-")) {
 		time = strsplit(s[0], "-");
-	} else {
-		time = {s[0]};		
+	//} else {
+	//	time = {s[0]};		
 	}
+	time.resize(3, "0");
 
-	if (time.size() == 1) {
-		return stoll(time[0]);
-	} else if (time.size() != 3) {
-		return 0;
-	}
+//	if (time.size() == 1) {
+//		return stoll(time[0]);
+//	} else if (time.size() != 3) {
+//		return 0;
+//	}
 
 	if (s.size() > 1) {
 		std::vector<std::string> secs = strsplit(s[1], ":");
